import os


def get_features(args, accounts) -> list:
    """
    Split the features as evenly as possible
    :param args:
    :param accounts:
    :return:
    """
    feature_data = [
        f'section.{f.split()[0]}' for f in
        os.listdir(os.path.join(args.dir)) if f.endswith('.feature')
    ]
    inc = -(-len(feature_data) // args.processes)  # weird, yucky
    features = [feature_data[i:i + inc] for i in range(0, len(feature_data), inc)]
    return list(zip(accounts, features))



import argparse


def parse_arguments():
    """
    Parses commandline arguments
    :return: Parsed arguments
    """
    parser = argparse.ArgumentParser('Running in parallel mode')

    # parallelism
    parser.add_argument('--processes', '-p',   required=True, type=int,  help='Maximum number of processes.', default=1)

    # total results/output
    parser.add_argument('--output_dir', '-of', required=True, type=str,  help='path to temporary output file', default='output')
    parser.add_argument('--zip_output', '-zo', help='toggle for zipping output', action='store_true')
    parser.add_argument('--save_output', '-so', help='toggle for removing output', action='store_true', default=False)

    # direct behave arguments (every bpp behave project requires these
    parser.add_argument('--env',       '-e',  required=True, type=str,  help='Environment to use for test run(s)', default=[], nargs='+')
    parser.add_argument('--res',       '-r',  required=True, type=str,  help='File path for aggregate results')
    parser.add_argument('--dir',       '-d',  required=True, type=str,  help='Path to a behave-like directory')
    parser.add_argument('--headless',  '-hd', help='toggle for headless browser', action='store_true')
    parser.add_argument('--browser',   '-b',  type=str,  help='Browser to use')
    parser.add_argument('--itags',     '-it', type=str,  help='Please specify behave tags to run', nargs='+')
    parser.add_argument('--etags',     '-et', type=str,  help='Please specify behave tags to exclude', default=[], nargs='+')
    parser.add_argument('--retry',     '-rt', type=int,  help='Max number of tries')

    # accounts arguments
    parser.add_argument('--account_file',    '-a',  required=True, type=str,  help='path to accounts ini file')
    parser.add_argument('--account_section', '-s',  required=True, type=str,  help='path to accounts ini file')

    # arbitrary project specific args (very breakable)
    parser.add_argument('--project_specific', nargs='+')
    return parser.parse_args()



# __main__.py
import os
import sys
import multiprocessing
from datetime import datetime
from multiprocessing import Pool
import shutil


# custom
from . import account_splitter
from . import feature_splitter
from . import arg_parser
from .parallelizer import command_generator
from . import results

ANUBIS_ASCII = (
    """
                    @@@                                               
                    @@@                                                 
                  @@ @@&@                                               
             @@@@@@@@@@@@                                               
                   # @,@@(                                              
                      @@@@@                                             
                   @@@@@@@@@                                            
                  @@@@@@@@@@@@@@@        @@@@@@                         
                  @@@@@@@@@@@@@@@@@@@@@@@@@@@% @@@@                     
                  @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@                   
              @@@@@@@@@@@@@@@@@@@@@@@@@     @@@@@@@@@@                  
    @@@@@@@@@@@@@@@@@@@       *@@@@     @@@@@@@@@@@@@@@@@@                 
                                                        @@              
    POWERED BY ANUBIS                                    @@@@            
                                                         @@@@@          
                                                          @@@@&         
                                                           @@@@         
                                                              @@      
    """
)


def main():
    print(ANUBIS_ASCII)
    start = datetime.now()

    # parse arguments
    arguments = arg_parser.parse_arguments()

    # create a temp dir that will contain results and be exported
    output_path = os.path.join('.', arguments.output_dir)
    if os.path.isdir(output_path):
        shutil.rmtree(output_path)
    os.mkdir(output_path)

    # set up the multiple processes
    # todo - set spawn method based on os (macos -> fork; others -> default)
    multiprocessing.set_start_method('fork')
    pool = Pool(arguments.processes)

    # get account data available for parallel runs
    print('--- Parsing accounts')
    print(f'\tfile:          <{arguments.account_file}>')
    print(f'\tsection:       <{arguments.account_section}>')
    accounts_data = account_splitter.get_accounts(
        arguments.processes,
        arguments.account_file,
        arguments.account_section
    )

    # split up the features and store as list
    print('--- Grouping features')
    print(f'\tfeature dir:   <{arguments.dir}>')
    print(f'\tincluded tags: <{",".join([t for t in arguments.itags])}>')
    print(f'\texcluded tags: <{",".join([t for t in arguments.etags])}>')
    account_feature_groups = feature_splitter.get_features(arguments, accounts_data)

    # run all the processes and save the locations of the result files
    print('--- Parallelizing')
    print(f'\tnum processes: <{len(account_feature_groups)}>')
    result_files = pool.map(command_generator, account_feature_groups)

    # recombine everything
    print('---RECOMBINING RESULTS')
    try:
        results.create_aggregate(
            files=result_files,
            aggregate_out_file=arguments.res
        )
    except Exception as e:
        print(e)

    # zip output if required
    if arguments.zip_output:
        results.zipper(output_path)

    # if arguments.sync_to:
    #     print(f'TODO: implement syncing to {arguments.sync_to}')
    #
    # if arguments.notify_to:
    #     print(f'TODO: implement notifying {arguments.notify_to}')

    if not arguments.save_output:
        shutil.rmtree(output_path)

    end = datetime.now()

    # extremely basic summary
    print('\n========================================')
    print(f'Env(s):   <{",".join(arguments.env)}>')
    print(f'Browser:  <{arguments.browser}>')
    print(f'Results:  <{arguments.res}>')
    print(f'Run Time: <{(end - start)}>')
    print('========================================')


if __name__ == '__main__':
    # run everything
    main()
    sys.exit(0)
